import os
import urllib.request
import json
from passlib.hash import pbkdf2_sha512
from urllib.parse import quote, urlencode
from requests_oauthlib import OAuth2Session
from requests_oauthlib.compliance_fixes import facebook_compliance_fix
from phanterpwa.backend.decorators import (
    requires_no_authentication
)
from phanterpwa.backend.dataforms import FieldsDALValidateDictArgs
from phanterpwa.i18n import browser_language

from phanterpwa.gallery.integrationDAL import PhanterpwaGalleryUserImage
from tornado import (
    web
)
from phanterpwa.third_parties.xss import xssescape as E
from phanterpwa.backend.security import (
    Serialize,
    SignatureExpired,
    BadSignature,
    URLSafeSerializer,
)

from datetime import (
    datetime
)
from phanterpwa.helpers import (
    HTML,
    HEAD,
    BODY,
    SCRIPT
)
from phanterpwa.backend.decorators import (
    check_client_token
)

class Errors(web.RequestHandler):
    """
        url: url: 'api/errors/'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "phanterpwa-authorization,",
                "cache-control"
            ])
        )
        self.set_header('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0')
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST')
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = self.request.headers.get("X-Real-IP") or \
            self.request.headers.get("X-Forwarded-For") or \
            self.request.remote_ip
        self.phanterpwa_origin = self.request.headers.get('Origin')

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})

    @check_client_token()
    def post(self, *args, **kargs):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        email_user = dict_arguments.get("email_user", None)
        current_way = dict_arguments.get("current_way", None)
        message = dict_arguments.get("message", None)
        error = dict_arguments.get("error", None)
        source = dict_arguments.get("source", None)
        lineno = dict_arguments.get("lineno", None)
        colno = dict_arguments.get("colno", None)
        if error:
            self.logger_api.error("CLIENT ERROR: {0}\n\t{1}\n\t{2}\n\t{3}\n\t{4}\nt{5}\nt{6}\n".format(
                error,
                "{0}: {1}".format("Email user", email_user),
                "{0}: {1}".format("Current way", current_way),
                "{0}: {1}".format("Message", message),
                "{0}: {1}".format("File", source),
                "{0}: {1}".format("Line", lineno),
                "{0}: {1}".format("Col", colno),

            )
        )

        message = "Received error."
        self.set_status(200)
        return self.write({
            'status': 'OK',
            'code': 200,
            'message': message,
            'i18n': {
                'message': self.T(message)
            }
        })





# -*- coding: utf-8 -*-
# author: PhanterJR<junior.conex@gmail.com>
# license: MIT

__author__ = "PhanterJR<junior.conex@gmail.com>"
__version__ = "14.4.2"
__install_requeriments__ = [
    "psutil",
    "tornado",
    "libsass",
    "transcrypt==3.9.0",
    "pillow",
    "itsdangerous==2.1.2",
    "requests-oauthlib",
    "pydal>=19.04",
    "passlib",
    "appdirs>=1.4.3"
]



# -*- coding: utf-8 -*-
# author: PhanterJR<junior.conex@gmail.com>
# license: MIT
import os
import sys
import psutil
import subprocess
import argparse
import traceback

from phanterpwa.tools import (
    config
)
from phanterpwa.compiler import Compiler
#from phanterpwa.interface.graphic import start
try:
    import tkinter as Tk
    from phanterpwa.interface.admin_tk import start
except:
    def start():
        print("The tkinter fails. Your Python may not be configured for Tk")
from phanterpwa.interface.cli import Cli
CURRENT_DIR = os.path.normpath(os.path.join(os.path.dirname(__file__)))
CWD = os.path.join(CURRENT_DIR, "interface")
CONFIG = config(CWD)
ENV_PYTHON = os.path.normpath(sys.executable)
parser = argparse.ArgumentParser(description="".join(['Without arguments the phanterpwa GUI will be executed,\n',
    ' if there is an error try to use the CLI version.']))


def start_server(projectPath):
    target = os.path.normpath(os.path.join(CURRENT_DIR, "server.py"))
    command = " ".join([ENV_PYTHON, target])
    subprocess.run(command, cwd=projectPath, shell=True)


def CLEAR_CONSOLE():
    os.system('cls' if os.name == 'nt' else 'clear')


if not len(sys.argv) > 1:
    def main():
        Cli()
else:
    parser.add_argument('-g', '--gui', action="store_true", default=False,
                        help='Grafical interface')

    parser.add_argument('-r', '--compile_and_server_last_project', action="store_true", default=False,
                        help='Compile and server of last running project')

    parser.add_argument('-s', '--server_last_project', action="store_true", default=False,
                        help='Run server of last running project')

    def main():
        args = parser.parse_args()
        if args.gui:
            start()
        elif args.compile_and_server_last_project or args.server_last_project:
            if 'last_application' in CONFIG:
                configProject = config(CONFIG['last_application'])
                projectPath = CONFIG['last_application']

                if args.compile_and_server_last_project:
                    try:
                        Compiler(projectPath).compile()
                    except Exception as e:
                        traceback.print_tb(e.__traceback__)

                print("API Server running in http://{0}:{1}".format(
                    configProject['API_SERVER']['host'], configProject['API_SERVER']['port']))
                print("APP Server running in http://{0}:{1}".format(
                    configProject['APP_SERVER']['host'], configProject['APP_SERVER']['port']))
                try:
                    print("Press CTRL+C to stop server!")
                    config(CWD, {'last_application': configProject['PATH']['project']})
                    start_server(projectPath)
                except KeyboardInterrupt:
                    target = os.path.abspath(os.path.join("..", "server.py"))
                    for p in psutil.process_iter():
                        cmd_line = None
                        try:
                            cmd_line = p.cmdline()
                        except Exception:
                            pass
                        if cmd_line:
                            if ENV_PYTHON == cmd_line[0]:
                                if os.path.normpath(target) == cmd_line[-1]:
                                    p.terminate()
                    CLEAR_CONSOLE()
                    input("Server Stoped""! Press <enter> key to exit.")
                    CLEAR_CONSOLE()
                except Exception as e:
                    traceback.print_tb(e.__traceback__)
                    input("Server error, check the log file to learn more. Press <enter> key to exit.")
                    CLEAR_CONSOLE()
        else:
            print("Try --cli or -c arguments")
        return True
if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
import unittest
import os
from phanterpwa.helpers import (
    # DIV,
    # IMG,
    # XML,
    # A,
    # TD,
    # SPAN,
    # CONCATENATE,
    # B,
    HTML,
    HEAD,
    BODY,
    SCRIPT,
    TITLE,
    META,
    STYLE,
    MAIN
)

CURRENT_PATH = os.path.dirname(__file__)

SAMPLE_HTML = HTML(
    HEAD(
        TITLE(
            'Test Components Preloaders'
        ),
        META(
            _charset='utf-8'
        ),
        META(
            _name='viewport',
            _content='width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no'
        ),
        META(
            _name='msapplication-tap-highlight',
            _content='no'
        ),
        SCRIPT(
            _src='/static/jquery.min.js'
        ),
        STYLE(
            "{{style}}"
        )
    ),
    BODY(
        MAIN(
            "{{content}}",
            _style="text-align: center;"
        ),
    ),
    _lang='pt-BR'
)


class TestComponentsPreloaders(unittest.TestCase):
    def test1_android(self):
        from phanterpwa.components.preloaders import android
        instanceAndroid = android.PRELOADER
        SAMPLE_HTML.html(
            formatter={'style': instanceAndroid.css(), 'content': instanceAndroid},
            file=os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_android.html")
        )
        self.assertTrue(os.path.isfile(os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_android.html")))

    def test2_square(self):
        from phanterpwa.components.preloaders import square
        instancesquare = square.PRELOADER
        SAMPLE_HTML.html(
            formatter={'style': instancesquare.css(), 'content': instancesquare},
            file=os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_square.html")
        )
        self.assertTrue(os.path.isfile(os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_square.html")))

    def test3_squares(self):
        from phanterpwa.components.preloaders import squares
        instancesquares = squares.PRELOADER
        SAMPLE_HTML.html(
            formatter={'style': instancesquares.css(), 'content': instancesquares},
            file=os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_squares.html")
        )
        self.assertTrue(os.path.isfile(os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_squares.html")))

    def test4_explosion(self):
        from phanterpwa.components.preloaders import explosion
        instanceexplosion = explosion.PRELOADER
        SAMPLE_HTML.html(
            formatter={'style': instanceexplosion.css(), 'content': instanceexplosion},
            file=os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_explosion.html")
        )
        self.assertTrue(os.path.isfile(os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_explosion.html")))

    def test5_run_points(self):
        from phanterpwa.components.preloaders import run_points
        instancerun_points = run_points.PRELOADER
        SAMPLE_HTML.html(
            formatter={'style': instancerun_points.css(), 'content': instancerun_points},
            file=os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_run_points.html")
        )
        self.assertTrue(os.path.isfile(os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_run_points.html")))

    def test5_discs(self):
        from phanterpwa.components.preloaders import discs
        instancediscs = discs.PRELOADER
        SAMPLE_HTML.html(
            formatter={'style': instancediscs.css(), 'content': instancediscs},
            file=os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_discs.html")
        )
        self.assertTrue(os.path.isfile(os.path.join(CURRENT_PATH, "preloaders", "test_components_preloaders_discs.html")))


if __name__ == '__main__':
    unittest.main()



import os
import io
from PIL import Image as PILImage, ImageOps


class PhanterpwaGalleryCutter(object):

    def __init__(self, imageName, imageBytes, cutterSizeX, cutterSizeY, force_png=False, trim_transparency_on_cutter=False):
        self.imageBytes = imageBytes
        self.cutterSizeX = cutterSizeX
        self.cutterSizeY = cutterSizeY
        self.force_png = force_png
        self.trim_transparency_on_cutter = trim_transparency_on_cutter
        if isinstance(imageBytes, bytes):
            self._img = PILImage.open(io.BytesIO(imageBytes))
            self._img = ImageOps.exif_transpose(self._img)
        else:
            self._img = PILImage.open(imageBytes)
            self._img = ImageOps.exif_transpose(self._img)
        if self._img.mode == "P":
            self._img = PILImage.open(imageBytes).convert("RGBA")
            self.content_type = "image/png"
            self._format = "PNG"
            self.extension = "png"
        elif self._img.mode == "RGBA" or force_png:
            self.content_type = "image/png"
            self._format = "PNG"
            self.extension = "png"
        else:
            self.content_type = "image/jpeg"
            self._format = "JPEG"
            self.extension = "jpg"
        self.image_name = imageName
        self.data = None

    @property
    def cutterSizeX(self):
        return self._cutterSizeX

    @cutterSizeX.setter
    def cutterSizeX(self, value):
        try:
            value = int(float(value))
        except Exception:
            raise ValueError("The cutterSizeX must be intenger. Given: {0}".format(type(value)))
        else:
            self._cutterSizeX = value

    @property
    def cutterSizeY(self):
        return self._cutterSizeY

    @cutterSizeY.setter
    def cutterSizeY(self, value):
        try:
            value = int(float(value))
        except Exception:
            raise ValueError("The cutterSizeY must be intenger. Given: {0}".format(type(value)))
        else:
            self._cutterSizeY = value

    @property
    def positionX(self):
        return self._positionX

    @positionX.setter
    def positionX(self, value):
        try:
            value = int(float(value))
        except Exception:
            raise ValueError("The positionX must be intenger. Given: {0}".format(type(value)))
        else:
            self._positionX = value

    @property
    def positionY(self):
        return self._positionY

    @positionY.setter
    def positionY(self, value):
        try:
            value = int(float(value))
        except Exception:
            raise ValueError("The positionY must be intenger. Given: {0}".format(type(value)))
        else:
            self._positionY = value

    @property
    def newSizeX(self):
        return self._newSizeX

    @newSizeX.setter
    def newSizeX(self, value):
        try:
            value = int(float(value))
        except Exception:
            raise ValueError("The newSizeX must be intenger. Given: {0}".format(type(value)))
        else:
            self._newSizeX = value

    @property
    def newSizeY(self):
        return self._newSizeY

    @newSizeY.setter
    def newSizeY(self, value):
        try:
            value = int(float(value))
        except Exception:
            raise ValueError("The newSizeY must be intenger. Given: {0}".format(type(value)))
        else:
            self._newSizeY = value

    @property
    def force_png(self):
        return self._force_png

    @force_png.setter
    def force_png(self, value):
        if isinstance(value, bool):
            self._force_png = value
        else:
            raise ValueError("The force_png must be boolean. Given: {0}".format(type(value)))

    @property
    def image_name(self):
        return self._image_name

    @image_name.setter
    def image_name(self, image_name):
        if "." in os.path.basename(image_name):
            self._image_name = "{0}.{1}".format(
                "".join(os.path.basename(image_name).split(".")[:-1]),
                self.extension
            )

    def specific_cut(self, newSizeX, newSizeY, positionX, positionY):
        img = self._img
        cutterSizeX = self.cutterSizeX
        cutterSizeY = self.cutterSizeY
        self.newSizeX = newSizeX
        self.newSizeY = newSizeY
        self.positionX = positionX
        self.positionY = positionY
        img = img.crop((self.positionX, self.positionY,
                      self.positionX + self.newSizeX,
                      self.positionY + self.newSizeY))
        if self.trim_transparency_on_cutter:
            img = img.crop(img.getbbox())
            img = ImageOps.contain(img, (max([cutterSizeX, cutterSizeY]), max([cutterSizeX, cutterSizeY])))
        else:
            try:
                img = img.resize((cutterSizeX, cutterSizeY),
                               PILImage.LANCZOS)
            except AttributeError:
                img = img.resize((cutterSizeX, cutterSizeY),
                               PILImage.ANTIALIAS)
        img_buffer = io.BytesIO()
        if self._format == "PNG":
            img.save(img_buffer, format=self._format)
        else:
            img.save(img_buffer, format=self._format, quality=100)
        img_buffer.seek(0)
        data = img_buffer.read()
        self.data = data
        return [self.image_name, data, self.content_type]

    def auto_cut(self, crop_type='middle'):
        crop_t = ["top", "middle", "bottom"]
        if crop_type not in crop_t:
            raise ValueError("".join(["The crop_type must be top, middle or bottom",
                " (default: middle). Given: ", str(crop_type)]))
        img = self._img
        img_ratio = img.size[0] / float(img.size[1])
        ratio = self.cutterSizeX / float(self.cutterSizeY)
        newSizeX = self.cutterSizeX * (img.size[1] / self.cutterSizeY)
        newSizeY = self.cutterSizeY * (img.size[0] / self.cutterSizeX)
        if ratio > img_ratio:
            if crop_type == 'top':
                box = (0, 0, img.size[0], newSizeY)
            elif crop_type == 'middle':
                box = (0, (img.size[1] - newSizeY) / 2, img.size[0], (img.size[1] + newSizeY) / 2)
            elif crop_type == 'bottom':
                box = (0, img.size[1] - newSizeY, img.size[0], img.size[1])
            img = img.crop(box)

        elif ratio < img_ratio:
            if crop_type == 'top':
                box = (0, 0, newSizeX, img.size[1])
            elif crop_type == 'middle':
                box = ((img.size[0] - newSizeX) / 2, 0, (img.size[0] + newSizeX) / 2, img.size[1])
            elif crop_type == 'bottom':
                box = (img.size[0] - newSizeX, 0, img.size[0], img.size[1])
            img = img.crop(box)
        if self.trim_transparency_on_cutter:
            img = img.crop(img.getbbox())
            img = ImageOps.contain(img, (max([self.cutterSizeX, self.cutterSizeY]), max([self.cutterSizeX, self.cutterSizeX])))
        else:
            try:
                img = img.resize((self.cutterSizeX, self.cutterSizeY),
                    PILImage.LANCZOS)
            except AttributeError:
                img = img.resize((self.cutterSizeX, self.cutterSizeY),
                    PILImage.ANTIALIAS)

        img_buffer = io.BytesIO()
        if self._format == "PNG":
            img.save(img_buffer, format=self._format)
        else:
            img.save(img_buffer, format=self._format, quality=100)

        img_buffer.seek(0)
        data = img_buffer.read()
        self.data = data
        return [self.image_name, data, self.content_type]

    def save(self, path):
        if os.path.exists(path) and os.path.isdir(path):
            if not self.data:
                self.auto_cut()
            with open(os.path.join(path, self.image_name), "wb") as g:
                g.write(self.data)
        else:
            raise ValueError("The path not exists or is not a dir")



import re
import json
from tornado import (
    websocket
)
from phanterpwa.backend.security import (
    Serialize,
    SignatureExpired,
    BadSignature,
    URLSafeSerializer,
)
check_compilation = re.compile(r"check_compilation\[([0-9]*)\]")

MSG = u"""______  _                    _               ______  _    _   ___  
| ___ \| |                  | |              | ___ \| |  | | / _ \ 
| |_/ /| |__    __ _  _ __  | |_   ___  _ __ | |_/ /| |  | |/ /_\ \\
|  __/ | '_ \  / _` || '_ \ | __| / _ \| '__||  __/ | |/\| ||  _  |
| |    | | | || (_| || | | || |_ |  __/| |   | |    \  /\  /| | | |
\_|    |_| |_| \__,_||_| |_| \__| \___||_|   \_|     \/  \/ \_| |_/

"""
class EchoWebSocket(websocket.WebSocketHandler):
    _connections = set()
    _online_users = {}

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.phanterpwa_current_user = None

    def check_origin(self, origin):
        return True

    def open(self):
        self.write_message(MSG)
        self.phanterpwa_current_user = None
        self.add_connection(self)

    def on_message(self, message):
        if message.startswith("{"):
            msg = None
            try:
                json_message = json.loads(message)
            except Exception as e:
                return self.write_message("JSON ERROR")
            else:
                id_user = None
                if "phanterpwa-authorization" in json_message and "message" in json_message:
                    msg = json_message["message"]
                    if json_message["phanterpwa-authorization"] is not "anonymous":
                        t = Serialize(
                            self.projectConfig['BACKEND'][self.app_name]['secret_key'],
                            self.projectConfig['BACKEND'][self.app_name]['default_time_user_token_expire']
                        )
                        token_content = None
                        try:
                            token_content = t.loads(json_message['phanterpwa-authorization'])
                        except BadSignature:
                            token_content = None
                        except SignatureExpired:
                            token_content = None
                        if token_content and 'id' in token_content:
                            id_user = token_content['id']
                    if id_user:
                        q_user = self.DALDatabase(self.DALDatabase.auth_user.id == id_user).select().first()
                        if q_user:
                            self.phanterpwa_current_user = q_user
                            try:
                                new_set = set(self._online_users[self.phanterpwa_current_user.id])
                            except KeyError:
                                new_set = set([self])
                            else:
                                new_set.add(self)
                            to_remove = []
                            for con in new_set:
                                if not con.ws_connection:
                                    to_remove.append(con)
                            for con in to_remove:
                                new_set.remove(con)
                            self._online_users[self.phanterpwa_current_user.id] = new_set

                            if msg == "command_online":
                                self.write_message(u"__ You're online")
                                return

            if message[:17] == "check_compilation":
                r = check_compilation.findall(message)
                if r:
                    app_compilation = r[0]
                    if str(self.projectConfig['PROJECT']['compilation']) == app_compilation:
                        self.write_message("Compilation: {0}".format(app_compilation))
                    else:
                        self.write_message("reload")
            else:
                self.write_message(u"You said: " + message)

    @classmethod
    def get_online_users(cls):
        user_cons = cls.get_connections()
        new_dict = dict()
        for con in user_cons:
            if con.phanterpwa_current_user and con.ws_connection:
                try:
                    new_set = set(new_dict[con.phanterpwa_current_user.id])
                except KeyError:
                    new_set = set()
                new_set.add(con)
                new_dict[con.phanterpwa_current_user.id] = new_set
        return new_dict

    @classmethod
    def check_user_is_online(cls, id_user):
        if id_user in cls.get_online_users():
            return True
        else:
            return False

    @classmethod
    def get_connections(cls):
        return tuple(cls._connections)

    @classmethod
    def add_connection(cls, connection):
        cls._connections.add(connection)

    @classmethod
    def remove_connection(cls, connection):
        try:
            cls._connections.remove(connection)
        except KeyError:
            pass

    def on_close(self):
        if self.phanterpwa_current_user:
            try:
                del self._online_users[self.phanterpwa_current_user.id]
            except KeyError:
                pass
        self.remove_connection(self)



import os
from datetime import datetime
from pydal import Field
from pydal.validators import (
    IS_IN_DB,
    IS_EMPTY_OR,
    IS_DATETIME
)


class pyDALTables():
    def __init__(self, db):
        self.db = db
        self.db.define_table('phanterpwagallery',
            Field('folder'),
            Field('filename'),
            Field('alias_name'),
            Field('content_type')
        )

        self.db.define_table('auth_user_phanterpwagallery',
            Field('phanterpwagallery', 'reference phanterpwagallery'),
            Field('auth_user', 'reference auth_user', requires=IS_EMPTY_OR(IS_IN_DB(self.db, self.db.auth_user))),
            Field('subfolder'),
            Field('last_update', 'datetime', default=datetime.now(), requires=IS_EMPTY_OR(IS_DATETIME()))
        )


class PhanterpwaGalleryUserImage():

    def __init__(self, id_user, db, projectConfig, appname="api"):
        self.db = db
        self.id_user = id_user
        self.upload_folder = os.path.normpath(
            os.path.join(projectConfig["PROJECT"]["path"], "backapps", appname, "uploads")
        )
        self.projectConfig = projectConfig

    @property
    def id_image(self):
        q_image = self.db(
            (self.db.auth_user_phanterpwagallery.auth_user == self.id_user) &
            (self.db.auth_user_phanterpwagallery.subfolder == 'profile') &
            (self.db.auth_user_phanterpwagallery.phanterpwagallery == self.db.phanterpwagallery.id)).select(
                self.db.auth_user_phanterpwagallery.id).last()
        if q_image:
            return q_image.id

    def alias_name(self, filename, id_image):
        ext = ""
        f_name = filename
        if "." in filename:
            ext = filename.split(".")[-1]
        f_name = "{0}.{1}".format(id_image, ext)
        return f_name

    def set_image(self, filename, file, content_type):
        db = self.db
        target_folder = os.path.join(self.upload_folder, "user_%s" % self.id_user, 'profile')
        os.makedirs(target_folder, exist_ok=True)
        q_image = self.db(
            (self.db.auth_user_phanterpwagallery.auth_user == self.id_user) &
            (self.db.auth_user_phanterpwagallery.subfolder == 'profile')).select(
                self.db.auth_user_phanterpwagallery.phanterpwagallery)
        if q_image:
            for q in q_image:
                t = os.path.join(os.path.join(target_folder, q.phanterpwagallery.alias_name))
                if os.path.exists(t):
                    try:
                        os.remove(t)
                    except OSError as e:
                        print("Error try delete {0}. Error: {1}".format(t, e))
                    else:
                        self.db(
                            self.db.phanterpwagallery.id == q.phanterpwagallery.id).delete()
            self.db.commit()
        id_new_image = self.db.phanterpwagallery.insert(
            folder=os.path.join("user_%s" % self.id_user, 'profile'),
            filename=filename,
            content_type=content_type)
        if id_new_image:
            alias_name = self.alias_name(filename, id_new_image)
            db.phanterpwagallery[id_new_image].update_record(alias_name=alias_name)

            with open(
                    os.path.join(target_folder, alias_name),
                    'wb') as new_image:
                new_image.write(file)
            new_vinculo = db.auth_user_phanterpwagallery.insert(
                phanterpwagallery=id_new_image,
                auth_user=self.id_user,
                subfolder="profile")
            if new_vinculo:
                self.db.commit()
                return new_vinculo

    @property
    def path_image(self):
        q_image = self.db(
            (self.db.auth_user_phanterpwagallery.auth_user == self.id_user) &
            (self.db.auth_user_phanterpwagallery.subfolder == 'profile') &
            (self.db.auth_user_phanterpwagallery.phanterpwagallery)).select(
                self.db.auth_user_phanterpwagallery.phanterpwagallery).last()
        if q_image:
            file = os.path.normpath(os.path.join(
                q_image.phanterpwagallery.folder,
                q_image.phanterpwagallery.alias_name
            ))
            if os.path.exists(file):
                return file


class PhanterpwaGalleryUpload(object):
    def __init__(self, db, _id=None):
        super(PhanterpwaGalleryUpload, self).__init__()
        self.db = db
        self.id = _id

    def insert_or_update(self, file, folder, filename, content_type):
        os.makedirs(folder, exist_ok=True)
        q_image = self.db(self.db.phanterpwagallery.id == self.id).select().first()
        if q_image:
            try:
                os.remove(os.path.join(folder, "{}".format(q_image.alias_name)))
            except OSError:
                pass
            new_alias_name = self.alias_name(filename, self.id)
            q_image.update_record(
                folder=folder,
                filename=filename,
                content_type=content_type,
                alias_name=new_alias_name
            )

            with open(os.path.join(folder, "{}".format(new_alias_name)), 'wb') as new_image:
                new_image.write(file)
            self.db.commit()
        else:
            id_image = self.db.phanterpwagallery.insert(
                folder=folder,
                filename=filename,
                content_type=content_type)
            if id_image:
                new_alias_name = self.alias_name(filename, id_image)
                self.db(self.db.phanterpwagallery.id == id_image).update(alias_name=new_alias_name)
                self.db.commit()
                self.id = id_image
                with open(
                        os.path.join(folder, "{}".format(new_alias_name)),
                        'wb') as new_image:
                    new_image.write(file)
        return self.id

    def alias_name(self, filename, id_image):
        ext = ""
        f_name = filename
        if "." in filename:
            ext = filename.split(".")[-1]
        f_name = "{0}.{1}".format(id_image, ext)
        return f_name

    def delete(self):
        q_image = self.db(self.db.phanterpwagallery.id == self.id).select().first()
        if q_image:
            try:
                os.remove(os.path.join(q_image.folder, "%s.%s" % (q_image.id, q_image.extensao)))
            except OSError:
                pass
            q_image.delete_record()
            self.db.commit()


class PhanterpwaGalleryImage():

    def __init__(self, sub_folder, db, projectConfig, id_image=0, appname="api"):
        self.db = db
        self.upload_folder = os.path.normpath(
            os.path.join(projectConfig["PROJECT"]["path"], "backapps", appname, "uploads")
        )
        self.projectConfig = projectConfig
        self.sub_folder = sub_folder
        self.id_image = id_image

    def alias_name(self, filename, id_image):
        ext = ""
        f_name = filename
        if "." in filename:
            ext = filename.split(".")[-1]
        f_name = "{0}.{1}".format(id_image, ext)
        return f_name

    def set_image(self, filename, file, content_type):
        db = self.db
        target_folder = os.path.join(self.upload_folder, self.sub_folder)
        os.makedirs(target_folder, exist_ok=True)
        rel_folder = os.path.join(self.sub_folder)
        if self.id_image:
            q_image = self.db(
                (self.db.phanterpwagallery.id == self.id_image)).select().first()
            if q_image:
                id_new_image = q_image.update_record(
                    folder=rel_folder,
                    filename=filename,
                    content_type=content_type
                )

            else:
                id_new_image = self.db.phanterpwagallery.insert(
                    folder=rel_folder,
                    filename=filename,
                    content_type=content_type
                )

        else:
            id_new_image = self.db.phanterpwagallery.insert(
                folder=rel_folder,
                filename=filename,
                content_type=content_type
            )

        self.db.commit()

        if id_new_image:
            self.id_image = id_new_image.id
            alias_name = self.alias_name(filename, self.id_image)
            q_img = db(db.phanterpwagallery.id == self.id_image).select().first()
            q_img.update_record(
                alias_name=alias_name
            )
            self.db.commit()
            with open(
                    os.path.join(target_folder, alias_name),
                    'wb') as new_image:
                new_image.write(file)
            return self.id_image

    @property
    def path_image(self):
        q_image = self.db(
            (self.db.phanterpwagallery.id == self.id_image)).select().last()
        if q_image:
            file = os.path.normpath(os.path.join(
                self.upload_folder,
                q_image.folder,
                q_image.alias_name
            ))
            if os.path.exists(file):
                return file


class PhanterpwaGalleryFile():

    def __init__(self, sub_folder, db, projectConfig, id_file=0, appname="api"):
        self.db = db
        self.upload_folder = os.path.normpath(
            os.path.join(projectConfig["PROJECT"]["path"], "backapps", appname, "uploads")
        )
        self.projectConfig = projectConfig
        self.sub_folder = sub_folder
        self.id_file = id_file

    def alias_name(self, filename, id_file):
        ext = ""
        f_name = filename
        if "." in filename:
            ext = filename.split(".")[-1]
        f_name = "{0}.{1}".format(id_file, ext)
        return f_name

    def set_file(self, filename, file, content_type):
        db = self.db
        target_folder = os.path.join(self.upload_folder, self.sub_folder)
        os.makedirs(target_folder, exist_ok=True)
        rel_folder = os.path.join(self.sub_folder)
        if self.id_file:
            s_file = self.db(
                (self.db.phanterpwagallery.id == self.id_file)).select().first()
            if s_file:
                id_new_file = s_file.update_record(
                    folder=rel_folder,
                    filename=filename,
                    content_type=content_type
                )

            else:
                id_new_file = self.db.phanterpwagallery.insert(
                    folder=rel_folder,
                    filename=filename,
                    content_type=content_type
                )

        else:
            id_new_file = self.db.phanterpwagallery.insert(
                folder=rel_folder,
                filename=filename,
                content_type=content_type
            )

        self.db.commit()

        if id_new_file:
            self.id_file = id_new_file.id
            alias_name = self.alias_name(filename, self.id_file)
            q_img = db(db.phanterpwagallery.id == self.id_file).select().first()
            q_img.update_record(
                alias_name=alias_name
            )
            self.db.commit()
            with open(
                    os.path.join(target_folder, alias_name),
                    'wb') as new_file:
                new_file.write(file)
            return self.id_file

    @property
    def path_file(self):
        s_file = self.db(
            (self.db.phanterpwagallery.id == self.id_file)).select().last()
        if s_file:
            file = os.path.normpath(os.path.join(
                self.upload_folder,
                s_file.folder,
                s_file.alias_name
            ))
            if os.path.exists(file):
                return file



import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.application as application
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = js_undefined = window = setTimeout = document = console = this = \
    __new__ = FormData = console = localStorage = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")

__pragma__('kwargs')


class Event():
    def __init__(self, identifier):
        self.identifier = identifier

    def reload(self):
        if window.PhanterPWA.DEBUG:
            console.info("The Event {0} reload".format(self.identifier))

    def start(self):
        if window.PhanterPWA.DEBUG:
            console.info("The Component {0} starts".format(self.identifier))


class WayHiperlinks(Event):
    def __init__(self):
        Event.__init__(self, "WayHiperlinks")

    def _open_way(self, el):
        window.PhanterPWA.open_way(jQuery(el).attr('phanterpwa-way'))
        window.PhanterPWA.Request.element = el


    def _bind(self, el, forced=False):
        el = jQuery(el)
        if not window.PhanterPWA.check_event_namespace(el, "click", "phanterpwa-way") or forced:
            el.off(
                "click.phanterpwa-way"
            ).on(
                "click.phanterpwa-way",
                lambda: self._open_way(this)
            )
            if window.PhanterPWA.DEBUG:
                console.info("Reload Event WayHiperlinks")

    def reload(self, **context):
        self.start(**context)

    def start(self, **context):
        target_selector = context.get("selector", "body")
        if jQuery("body").hasClass("phanterpwa-lock"):
            jQuery("[phanterpwa-way]").off("click.phanterpwa-way")
        else:
            jQuery(target_selector).find("[phanterpwa-way]").each(lambda: self._bind(this))


class Waves(Event):
    def __init__(self):
        Event.__init__(self, "Waves")

    def _on_click(self, el, ev):
        el = jQuery(el)
        pY = ev.pageY - el.offset().top
        pX = ev.pageX - el.offset().left

        wave = DIV(
            _class="wave"
        ).jquery()
        el.append(wave.css("left", pX).css("top", pY).animate({
            "top": pY - 300,
            "left": pX - 300,
            "width": 600,
            "height": 600,
            "opacity": 0
        }, 1000))
        setTimeout(lambda: wave.remove(), 2000)

    def reload(self, **context):
        self.start(**context)

    def _bind(self, el, forced=False):
        el = jQuery(el)
        if not window.PhanterPWA.check_event_namespace(el, "click", "phanterpwa-event-wave") or forced:
            el.off("click.phanterpwa-event-wave").on(
                "click.phanterpwa-event-wave",
                lambda event: self._on_click(this, event)
            )
            if window.PhanterPWA.DEBUG:
                console.info("Reload Event Wave")

    def start(self, **context):
        target_selector = context.get("target", "body")
        jQuery(target_selector).find(".wave_on_click").each(lambda: self._bind(this))


# class WidgetsInput(Event):
#     def __init__(self):
#         Event.__init__(self, "WidgetsInput")
#         self.focus = False
#         self.has_val = None

#     def reload(self):
#         for x in window.PhanterPWA.Request.widgets.keys():
#             window.PhanterPWA.Request.widgets[x]._reload()

#     def _cleck_element(self, instance):
#         if jQuery(instance.target_selector).length == 0:
#             return False
#         return True

#     def start(self):
#         for x in window.PhanterPWA.Request.widgets.keys():
#             window.PhanterPWA.Request.widgets[x].start()


__pragma__('nokwargs')



#?from org.transcrypt.stubs.browser import __pragma__
# __pragma__ ('ecom')
#?__pragma__('alias', "jQuery", "$")
#?__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = Hammer =\
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = Date = 0

#?__pragma__('noskip')


#?__pragma__('kwargs')


def requires_authentication(users_id=None, users_email=None, roles_id=None, roles_name=None):
    def decorator(f):

        def requires_authentication_decorator(self, *args, **kargs):
            self.status = 200
            if any([users_id is not None, users_email is not None, roles_id is not None, roles_name is not None]):
                if isinstance(users_id, (list, tuple)):
                    if self.current_user.id in users_id:
                        return f(self, *args, **kargs)
                if isinstance(users_id, int):
                    if self.current_user.id == users_id:
                        return f(self, *args, **kargs)
                if isinstance(users_email, (list, tuple)):
                    if self.current_user.email in users_email:
                        return f(self, *args, **kargs)
                if isinstance(users_email, str):
                    if users_email == self.current_user.email:
                        return f(self, *args, **kargs)
                if isinstance(self.current_user.roles, list):
                    if isinstance(roles_id, int):
                        for x in self.current_user.roles_id:
                            if x == roles_id:
                                return f(self, *args, **kargs)
                    if isinstance(roles_name, str):
                        for x in self.current_user.roles:
                            if x == roles_name:
                                return f(self, *args, **kargs)
                    if isinstance(roles_id, (list, tuple)):
                        for x in self.current_user.roles_id:
                            if x in roles_id:
                                return f(self, *args, **kargs)
                    if isinstance(roles_name, (list, tuple)):
                        for x in self.current_user.roles:
                            if x in roles_name:
                                return f(self, *args, **kargs)
                self.status = 403
                return self._error(403)
            elif self.current_user is None or self.current_user is js_undefined:
                self.status = 403
                return self._error(403)
            else:
                return f(self, *args, **kargs)

        return requires_authentication_decorator
    return decorator


def requires_no_authentication():
    def decorator(f):
        def requires_no_authentication_decorator(self, *args, **kargs):
            self.status = 200
            if self.phanterpwa_current_user is None or self.phanterpwa_current_user is js_undefined:
                return f(self, *args, **kargs)
            else:
                self.status = 403
                return self._error(403)
        return requires_no_authentication_decorator
    return decorator

def check_authorization(authorization_test=False, on_error=None):
    def decorator(f):
        def check_authorization_decorator(responseObj):
            test = False
            if callable(authorization_test):
                test = authorization_test()

            if test is True:
                window.PhanterPWA.Response = responseObj
                return f(responseObj)
            else:
                code = 401
                if window.PhanterPWA.logged():
                    code = 403

                if callable(on_error):
                    on_error(code,  responseObj._request, responseObj)
                else:
                    window.PhanterPWA.open_code_way(code, responseObj._request, responseObj)
                
        return check_authorization_decorator
    return decorator

#?__pragma__('nokwargs')
# __pragma__ ('noecom')


import phanterpwa.frontend.helpers as helpers
from org.transcrypt.stubs.browser import __pragma__


__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = js_undefined = setTimeout = window = this = console =\
    localStorage = 0
__pragma__('noskip')


DIV = helpers.XmlConstructor.tagger("div")
I = helpers.XmlConstructor.tagger("i")

__pragma__('kwargs')


class TopSlide(helpers.XmlConstructor):
    def __init__(self, target_selector, *content, **parameters):
        self.target_selector = target_selector
        self.element_target = jQuery(self.target_selector)
        self._after_open = None
        if "after_open" in parameters:
            self._after_open = parameters["after_open"]
        if "_id" not in parameters:
            parameters["_id"] = "phanterpwa-component-topslide-wrapper"
        if "_class" in parameters:
            parameters["_class"] = "{0}{1}".format(
                parameters["_class"].strip(),
                " phanterpwa-component-topslide-wrapper"
            )
        else:
            parameters["_class"] = "phanterpwa-component-topslide-wrapper"
        content.insert(0, DIV(I(_class="fas fa-times"), _class="phanterpwa-component-topslide-close link"))
        helpers.XmlConstructor.__init__(self, "div", False, *content, **parameters)

    def switch(self):
        self.element_target = jQuery(self.target_selector)
        if self.element_target.find("#phanterpwa-component-topslide-wrapper").hasClass("enabled"):
            self.close()
        else:
            self.open()

    def close(self):
        self.element_target = jQuery(self.target_selector)
        if self.element_target.find("#phanterpwa-component-topslide-wrapper").hasClass("enabled"):
            self.element_target.find("#phanterpwa-component-topslide-wrapper").removeClass("enabled")
            self.element_target.find("#phanterpwa-component-topslide-wrapper").slideUp()

    def open(self):
        self.element_target = jQuery(self.target_selector)
        self.start()
        if not self.element_target.find("#phanterpwa-component-topslide-wrapper").hasClass("enabled"):
            self.element_target.find("#phanterpwa-component-topslide-wrapper").addClass("enabled")
            self.element_target.find("#phanterpwa-component-topslide-wrapper").slideDown()
        if self._after_open is not None and self._after_open is not js_undefined:
            self._after_open(self.element_target.find("#phanterpwa-component-topslide-wrapper"))

    def start(self):
        self.element_target = jQuery(self.target_selector)
        if self.element_target is not None and self.element_target is not js_undefined:
            self.element_target.find('#phanterpwa-component-topslide-wrapper').remove()
            self.append_to(self.element_target)
            self.element_target.find(".phanterpwa-component-topslide-close").off(
                "click.phanterpwa_component_topslide_close"
            ).on(
                "click.phanterpwa_component_topslide_close",
                self.close
            )


__pragma__('nokwargs')



# -*- coding: utf-8 -*-
# author: PhanterJR<junior.conex@gmail.com>
# license: MIT

__author__ = "PhanterJR<junior.conex@gmail.com>"
__version__ = "1.0.0"

from ..helpers import DIV, I, INPUT, CANVAS, CONCATENATE
from ..xmlconstructor import XmlConstructor
import io
import os
from PIL import Image as PILImage, ImageOps


class PhanterpwaGalleryInput(XmlConstructor):

    def __init__(self, src_img="", cut_size=(300, 300), global_id=None, zindex=None, **attributes):
        """
        @src_img: source_img

        @cut_size: Cut image size

        @global_id: Adds in all ids of the elements this id, in this way it is
            possible to add 2 distinct inputs without conflicts
        """
        if "_class" in attributes:
            new_class = " ".join([attributes['_class'].strip(), "phanterpwa-gallery-inert-container"])
            attributes['_class'] = new_class
        else:
            attributes['_class'] = "phanterpwa-gallery-inert-container"
        self.cut_size = cut_size
        self.title_button = "Upload Image"
        self._image_button = I(_class="fas fa-cloud-upload-alt")
        self.global_id = global_id
        self._src_img = src_img
        self.zindex = zindex
        XmlConstructor.__init__(self, 'div', False, **attributes)

    @property
    def just_cutter_panel(self):
        self.xml()
        return self._just_cutter_panel

    @property
    def just_buttom(self):
        self.xml()
        return self._just_buttom

    @property
    def zindex(self):
        return self._zindex

    @zindex.setter
    def zindex(self, value):
        if isinstance(value, (int, type(None))):
            self._zindex = value
        else:
            raise TypeError("The zindex value must be integer")

    @property
    def global_id(self):
        return self._global_id

    @global_id.setter
    def global_id(self, _id):
        if isinstance(_id, (str, int, type(None))):
            if isinstance(_id, str):
                _id = _id.strip()
                if " " in _id:
                    raise ValueError("The id can't have empty space")
                else:
                    self._global_id = _id
            elif _id is None:
                self._global_id = ""
            else:
                self._global_id = _id
        else:
            raise TypeError("The global_id must be string or integer or None")

    @property
    def title_button(self):
        """
        Get or Set title button
        """
        return self._title_button

    @title_button.setter
    def title_button(self, title):
        if isinstance(title, (str, XmlConstructor)):
            if isinstance(title, XmlConstructor):
                title = title.xml()
            self._title_button = title
        else:
            raise TypeError("The title must be string")

    @property
    def image_button(self):
        """
        Get or Set title button
        """
        return self._image_button

    @image_button.setter
    def image_button(self, img_button):
        if isinstance(img_button, (str, XmlConstructor)):
            if isinstance(img_button, XmlConstructor):
                img_button = img_button.xml()
            self._image_button = img_button
        else:
            raise TypeError("The image_button must be " +
                            "string or XmlConstructor instance")

    def humanize(self):
        global_id = self._global_id
        self._html_input(global_id)
        human = ""
        space = " " * (self._ident_level * self._ident_size)
        if self.content and not self.void:
            human = "".join([self.tag_begin, self.xml_content_for_humans, "\n", space, self.tag_end])
        elif self.void:
            human = "".join([self.tag_begin])
        else:
            human = "".join([self.tag_begin, "\n", space, self.tag_end])
        return "".join(["\n", space, human]).replace('&#58;', ':')

    def xml(self):
        global_id = self._global_id
        self._html_input(global_id)
        xml = ""
        if self.content and not self.void:
            xml = "".join([self.tag_begin, self.xml_content, self.tag_end])
        elif self.void:
            xml = self.tag_begin
        else:
            xml = "".join([self.tag_begin, self.tag_end])
        return xml

    def _html_input(self, _id=""):
        zindex = self._zindex
        style_zindex = None
        if zindex:
            style_zindex = "z-index:%s;" % zindex
        title_button = self._title_button
        image_button = self._image_button
        cut_size = self.cut_size
        ids_elements = {
            '_data-object': 'phanterpwa-gallery_object',
            '_data-upload-form-container':
                'phanterpwa-gallery_upload-form-container',
            '_data-upload-input': 'phanterpwa-gallery_upload-input-file',
            '_data-cutter-pad':
                'phanterpwa-gallery_cutter-pad',
            '_data-cutter-background':
                'phanterpwa-gallery_cutter-background',
            '_data-panel-cutter-container':
                'phanterpwa-gallery_panel-cutter-container',
            '_data-cutter-shadow':
                'phanterpwa-gallery_cutter-shadow',
            '_data-cutter-control-close':
                'phanterpwa-gallery_cutter-control-close',
            '_data-cutter-control-view':
                'phanterpwa-gallery_cutter-control-view',
            '_data-cutter-control-cut':
                'phanterpwa-gallery_cutter-control-cut',
            '_data-panel-cutter-size-container':
                'phanterpwa-gallery_panel-cutter-size-container',
            '_data-panel-cutter-image':
                'phanterpwa-gallery_panel-cutter-image',
            '_data-cutter-zoom-control':
                'phanterpwa-gallery_cutter-zoom-control',
            '_data-target-view':
                'phanterpwa-gallery_target-view',
            '_data-target-view-container':
                'phanterpwa-gallery_target-view-container',
            '_data-upload-messages':
                'phanterpwa-gallery_upload-messages',
            '_data-upload-area-progress':
                'phanterpwa-gallery_upload-area-progress',
            '_data-upload-image-button':
                'phanterpwa-gallery_upload-image-button',
            '_data-upload-title-button':
                'phanterpwa-gallery_upload-title-button',
            '_data-imagecuted-control-erase':
                'phanterpwa-gallery-imagecuted-control-erase',
            '_data-imagecuted-control-change':
                'phanterpwa-gallery-imagecuted-control-change',
            '_data-input-name-cuttersizex':
                'phanterpwa-gallery-input-name-cuttersizex',
            '_data-input-name-cuttersizey':
                'phanterpwa-gallery-input-name-cuttersizey',
            '_data-input-name-positionx':
                'phanterpwa-gallery-input-name-positionx',
            '_data-input-name-positiony':
                'phanterpwa-gallery-input-name-positiony',
            '_data-input-name-newsizex':
                'phanterpwa-gallery-input-name-newsizex',
            '_data-input-name-newsizey':
                'phanterpwa-gallery-input-name-newsizey',
        }
        if _id:
            for x in ids_elements:
                ids_elements[x] = "-".join([ids_elements[x], _id])
        ids_elements['_data-cutter-size-x'] = str(cut_size[0])
        ids_elements['_data-cutter-size-y'] = str(cut_size[1])
        ids_elements['_data-upload-src-img'] = self._src_img
        self._just_cutter_panel = DIV(
            DIV(
                _id=ids_elements["_data-cutter-background"],
                _class="phanterpwa-gallery-cutter-background"),
            DIV(
                _id=ids_elements["_data-cutter-shadow"],
                _class="phanterpwa-gallery-cutter-shadow"),
            DIV(
                DIV(
                    DIV(
                        _class="phanterpwa-gallery-panel-cutter-image",
                        _id=ids_elements['_data-panel-cutter-image']),

                    _style="overflow: hidden; width: %spx; height: %spx;" %
                           (cut_size[0], cut_size[1]),
                    _id=ids_elements['_data-panel-cutter-size-container'],
                    _class="phanterpwa-gallery-panel-cutter-size-container"
                ),
                _class="phanterpwa-gallery-panel-cutter"
            ),
            DIV(
                _id=ids_elements['_data-cutter-pad'],
                _class="phanterpwa-gallery-cutter-pad"),
            DIV(
                DIV(
                    I(_class="phanterwebsvg close-circle"),
                    _id=ids_elements['_data-cutter-control-close'],
                    _class="phanterpwa-gallery-cutter-control"),
                DIV(
                    I(_class="phanterwebsvg image-view"),
                    _id=ids_elements['_data-cutter-control-view'],
                    _class="phanterpwa-gallery-cutter-control"),
                DIV(
                    I(_class="phanterwebsvg image-cut"),
                    _id=ids_elements['_data-cutter-control-cut'],
                    _class="phanterpwa-gallery-cutter-control"),
                _class='phanterpwa-gallery_cutter-controls-container'),
            DIV(
                DIV(
                    I(_class="phanterwebsvg image-decrease"),
                    DIV(
                        DIV(_id=ids_elements['_data-cutter-zoom-control'],
                            _class="phanterpwa-gallery-cutter-zoom-control"),
                        _class="phanterpwa-gallery-cutter-zoom-control" +
                               "-container"
                    ),
                    I(_class="phanterwebsvg image-increase"),
                    _class='phanterpwa-gallery_cutter-zoom-controls'),
                _class='phanterpwa-gallery_cutter-zoom-container'),
            _id=ids_elements['_data-panel-cutter-container'],
            _class="phanterpwa-gallery-panel-cutter-container",
            _style=style_zindex
        )
        self._just_buttom = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV(
                            I(_class="phanterwebsvg recycle"),
                            _id=ids_elements["_data-imagecuted-control-erase"],
                            _class="phanterpwa-gallery" +
                                   "_imagecuted-control"
                        ),
                        DIV(
                            I(_class="phanterwebsvg image-reload"),
                            _id=ids_elements["_data-imagecuted" +
                                             "-control-change"],
                            _class="phanterpwa-gallery" +
                                   "_imagecuted-control"
                        ),
                        _class="phanterpwa-gallery" +
                               "_imagecuted-controls"),
                    CANVAS(_id=ids_elements['_data-target-view']),
                    _class='phanterpwa-gallery-center-content',
                ),
                _id=ids_elements['_data-target-view-container'],
                _style="overflow: hidden; width: %spx; height: %spx;" %
                       (cut_size[0], cut_size[1]),
                _class="phanterpwa-gallery-target-view-container%s" %
                       (" actived" if self._src_img else ""),
            ),
            DIV(
                DIV(
                    DIV(
                        DIV(image_button,
                            _id=ids_elements["_data-upload-image-button"],
                            _class="phanterpwa-gallery-upload-image-button"),
                        DIV(title_button,
                            _id=ids_elements["_data-upload-title-button"],
                            _class="phanterpwa-gallery-upload-title-button"),
                        _class="phanterpwa-gallery-upload-button"
                    ),
                    _class="phanterpwa-gallery-container-upload-button"
                ),
                _id=ids_elements['_data-object'],
                _class="phanterpwa-gallery-object%s" %
                       (" actived" if not self._src_img else ""),
                **ids_elements
            ),
            DIV(
                DIV(
                    INPUT(
                        _accept="image/png, image/jpeg, image/gif, image/bmp",
                        _id=ids_elements["_data-upload-input"],
                        _class="phanterpwa-gallery-upload-input-file",
                        _type="file",
                        _name=ids_elements["_data-upload-input"]
                    ),
                    _class="input-field"),
                INPUT(_id=ids_elements['_data-input-name-cuttersizex'],
                    _name=ids_elements['_data-input-name-cuttersizex'],
                    _value="",
                    _type="text"),
                INPUT(_id=ids_elements['_data-input-name-cuttersizey'],
                    _name=ids_elements['_data-input-name-cuttersizey'],
                    _value="",
                    _type="text"),
                INPUT(_id=ids_elements['_data-input-name-positionx'],
                    _name=ids_elements['_data-input-name-positionx'],
                    _value="",
                    _type="text"),
                INPUT(_id=ids_elements['_data-input-name-positiony'],
                    _name=ids_elements['_data-input-name-positiony'],
                    _value="",
                    _type="text"),
                INPUT(_id=ids_elements['_data-input-name-newsizex'],
                    _name=ids_elements['_data-input-name-newsizex'],
                    _value="",
                    _type="text"),
                INPUT(_id=ids_elements['_data-input-name-newsizey'],
                    _name=ids_elements['_data-input-name-newsizey'],
                    _value="",
                    _type="text"),
                _id=ids_elements["_data-upload-form-container"],
                _class="phanterpwa-gallery-upload-form-container",
                _style="display: none;"
            ),

            DIV(_id=ids_elements['_data-upload-messages'],
                _class="phanterpwa-gallery-upload-messages"),
            DIV(
                DIV(
                    DIV(
                        DIV(_class="phanterpwa-gallery-progressbar-movement"),
                        _class="phanterpwa-gallery-progressbar"),
                    _id=ids_elements['_data-upload-area-progress'],
                    _class="phanterpwa-gallery-upload-area-progress"
                ),
                _class="phanterpwa-gallery-progressbar-container")
        )
        html = CONCATENATE(self._just_buttom, self._just_cutter_panel)
        self.content = [html]

    @property
    def cut_size(self):
        return self._cut_size

    @cut_size.setter
    def cut_size(self, cut_size):
        if isinstance(cut_size, (tuple, list)):
            if len(cut_size) == 2:
                if isinstance(cut_size[0], (int, float)) and \
                        isinstance(cut_size[1], (int, float)):
                    self._cut_size = (cut_size[0], cut_size[1])
                else:
                    raise TypeError("Values of list or tuple must be " +
                                    "integers or float")
            else:
                raise SyntaxError("There must be at least 2 values")
        else:
            raise TypeError("The value has to be a tuple or a list")


class PhanterpwaGalleryCutter(object):

    def __init__(self,
                 imageName,
                 imageBytes,
                 cuttersizex, cuttersizey,
                 positionx, positiony,
                 newsizex, newsizey, force_png=False):
        self.imageBytes = imageBytes
        self.cuttersizex = cuttersizex
        self.cuttersizey = cuttersizey
        self.positionx = positionx
        self.positiony = positiony
        self.newsizex = newsizex
        self.newsizey = newsizey
        self.force_png = force_png
        if force_png:
            self.extensao = 'png'
        else:
            self.extensao = 'jpg'
        self.nome_da_imagem = imageName

    def getImage(self):
        imageBytes = self.imageBytes
        nome_da_imagem = self.nome_da_imagem
        im = PILImage.open(imageBytes)
        im = ImageOps.exif_transpose(im)
        newsizex = int(float(self.newsizex))
        newsizey = int(float(self.newsizey))
        cuttersizex = int(float(self.cuttersizex))
        cuttersizey = int(float(self.cuttersizey))
        positionx = float(self.positionx)
        positiony = float(self.positiony)
        nome_do_arquivo, extensao = os.path.splitext(nome_da_imagem)
        im = im.crop((positionx, positiony,
                      positionx + newsizex,
                      positiony + newsizey))
        im = im.resize((cuttersizex, cuttersizey),
                       PILImage.ANTIALIAS)
        jpeg_image_buffer = io.BytesIO()
        if extensao.lower() == '.png' or self.force_png:
            self.extensao = 'png'
            self.nome_da_imagem = "%s.%s" % (nome_do_arquivo, "png")
            im.save(jpeg_image_buffer, 'png')
        else:
            self.extensao = 'jpg'
            self.nome_da_imagem = "%s.%s" % (nome_do_arquivo, "jpg")
            im.save(jpeg_image_buffer, format='JPEG', quality=100)
        jpeg_image_buffer.seek(0)
        data = jpeg_image_buffer.read()
        return data


class PhanterpwaGalleryImageBytes(object):

    def __init__(self, imageName, imageBytes, force_png=False):
        self.nome_da_imagem = imageName
        self.imageBytes = imageBytes
        if force_png:
            self.extensao = 'png'
        else:
            self.extensao = 'jpg'

    def getImage(self, formatOut='png'):
        nome_da_imagem = self.nome_da_imagem
        nome_do_arquivo, extensao = os.path.splitext(nome_da_imagem)
        im = PILImage.open(self.imageBytes)
        jpeg_image_buffer = io.BytesIO()
        if extensao.lower() == '.png' or self.force_png:
            self.extensao = 'png'
            self.nome_da_imagem = "%s.%s" % (nome_do_arquivo, "png")
            im.save(jpeg_image_buffer, 'png')
        else:
            self.extensao = 'jpg'
            self.nome_da_imagem = "%s.%s" % (nome_do_arquivo, "jpg")
            im.save(jpeg_image_buffer, format='JPEG', quality=100)
        jpeg_image_buffer.seek(0)
        data = jpeg_image_buffer.read()
        return data



# Created automatically.
#
# In development it may be necessary to add static data
# to the client side application after compiling, use
# the CONFIGJS section of the application's app.ini
# file for this.
#

CONFIG = {
    "PROJECT": {
        "name": "Admin",
        "title": "PhanterPWA",
        "version": "0.0.1",
        "compilation": 1,
        "debug": True,
        "author": "PhanterJR<phanterjr@conexaodidata.com.br>"
    },
    "APP": {
        "build_folder": "C:\\Virtualenv\\py37\\phanterpwaenv\\Lib\\site-packages\\phanterpwa\\interface\\Admin\\frontapps\\admin\\www",
        "title": "Admin",
        "transcrypt_main_file": "application",
        "styles_main_file": "application",
        "timeout_to_resign": 600,
        "http_address": "http://127.0.0.1:10000",
        "websocket_address": "ws://127.0.0.1:10000/api/websocket",
        "name": "admin"
    },
    "I18N": {}
}



# -*- coding: utf-8 -*-
import unittest
import os

from phanterpwa.reversexml import (
    HtmlToXmlConstructor,
    force_minify_string_content
)


sample_example_1 = """HTML(
    HEAD(
        META(
            _charset='UTF-8'
        )
    ),
    BODY(
        NAV(
            BUTTOM(
                'start'
            ),
            _class='navbar'
        ),
        MAIN(
            DIV(
                DIV(
                    'my content'
                ),
                _class='row'
            ),
            _id='my_content'
        )
    )
)"""
sample_example_2 = """CONCATENATE(
    DIV(
        'content1',
        **{
            '_data-dict': 'i am in dict',
            '_class': 'my_class'
        }
    ),
    DIV(
        'content2',
        _class='my_class'
    )
)"""

sample_html_to_xmlconstructor = """<div id="test02" enabled empty="=">
  <div style="background-color:blue; width:100%; height:40px;">
    OOOOOPAAAAA
    OBA
  </div>
  <div json='{"teste": true}' testando-outro='["macaco", true, "boi"]' texto_com_aspas="queria muito &quot;tentar&quot; ou não 'tentar'" list_none="[null]" dict='{"testesss": true}' json_dums="[null, true, false]">
    <div>
      conteudo
    </div>
  </div>
  <b>crazy_helpers</b>
  &lt;b&gt;crazy_helpers&lt;/b&gt;
</div>
<div>&lt;main&gt;<b>xsscapedthis</b>&lt;/main&gt;</div>
&lt;div&gt;&lt;main&gt;<b>xsscapedthis</b>&lt;/main&gt;&lt;/div&gt;
<div></div>"""

sample = '''
<html>
    <head>
        <meta charset="utf-8">
    </head>
    <body>
        Forces      the       minification
        of the     string content       by
        removing       multiple     empty
        spaces,      line       breaks,
        tabs      replacing     with a
        single    space.
    </body>
</html>
'''

sample2 = '''<html><head><meta charset="utf-8"></head><body> Forces the minification of the string content by removing multiple empty spaces, line breaks, tabs replacing with a single space. </body></html>'''

sample3 = '''HTML(
    HEAD(
        META(
            _charset='utf-8'
        )
    ),
    BODY(
        ' Forces the minification of the string content by removing multiple empty spaces, line breaks, tabs replacing with a single space. '
    )
)'''


class TestReverseXml(unittest.TestCase):

    def test0_documentation_examples(self):

        html_to_xmlconstructor_instance = HtmlToXmlConstructor("<html><head><meta charset=\"UTF-8\"></head><body>" +
            "<nav class=\"navbar\"><buttom>start</buttom></nav><main id=\"my_content\"><div class=\"row\"><div>my" +
            " content</div></div></main></body></html>")

        self.assertEqual(
            html_to_xmlconstructor_instance.xmlconstructor_code(),
            sample_example_1
        )
        html_to_xmlconstructor_instance = HtmlToXmlConstructor("<div data-dict=\"i am in dict\" class=\"my_class\">" +
            "content1</div><div class=\"my_class\">content2</div>")
        self.assertEqual(
            html_to_xmlconstructor_instance.xmlconstructor_code(),
            sample_example_2
        )
        html_to_xmlconstructor_instance = HtmlToXmlConstructor(sample)
        self.assertEqual(
            html_to_xmlconstructor_instance.xml(),
            sample
        )
        self.assertEqual(
            force_minify_string_content(html_to_xmlconstructor_instance).xml(),
            sample2
        )
        self.assertEqual(
            force_minify_string_content(html_to_xmlconstructor_instance).xmlconstructor_code(),
            sample3
        )

    def test1_html_to_xmlconstructor(self):
        invert = HtmlToXmlConstructor(sample_html_to_xmlconstructor)
        self.assertEqual(invert.xml(), sample_html_to_xmlconstructor)
        invert.src_attr_dict = False
        self.assertRaises(ValueError, lambda: invert.source_code())
        invert.src_attr_dict = None
        self.assertTrue(isinstance(invert.source_code(), str))
        invert.src_attr_dict = True
        self.assertTrue(isinstance(invert.source_code(), str))


if __name__ == '__main__':
    unittest.main()



from tornado import (
    web
)
from core import (
    projectConfig
)

class Welcome(web.RequestHandler):
    def get(self, *args):
        self.write({
            "status": "OK",
            "message": "Hello World!",
            "project": projectConfig["PROJECT"]
        })



"""
Title: XmlConstructor

Author: PhanterJR<junior.conex@gmail.com>

License: MIT

Coding: utf-8

Send emails
"""

import smtplib
from pydal.validators import IS_EMAIL
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart


class MailSender(object):
    """
    :param email_sender: Email used in sending
    :param password_sender: Email password used when sending
    :param email_receiver: Recipient email
    :param subject: Email title
    :param text_message: Plain text message
    :param html_message: Html format message
    :param alternative_sender: Reply email
    :param server: Server address
    :param port: Used port
    :param use_tls: Used tls
    :param use_ssl: Used ssl
    """

    def __init__(
            self,
            email_sender: str,
            password_sender: str,
            email_receiver: str,
            subject: str="Test",
            text_message: str="Test",
            html_message: str="<html><b>Test</b></html>",
            alternative_sender: (str, None)=None,
            server: str='127.0.0.1',
            port: int=25,
            use_tls: bool=False,
            use_ssl: bool=False):
        super(MailSender, self).__init__()
        self.email_sender = email_sender
        self.password_sender = password_sender
        self.email_receiver = email_receiver
        self.subject = subject
        self.text_message = text_message
        self.html_message = html_message
        self.alternative_sender = alternative_sender
        self.server = server
        self.port = port
        self.use_tls = use_tls
        self.use_ssl = use_ssl

    @property
    def email_sender(self) -> str:
        """Get ot Set the email sender.

        :GET:

            Get the email sender

        :SET:
            When adding an email, it is validated, if it is not a valid email it returns a ValueError

        Example:
            >>> my_instance = MailSender(
            ...     "sender@email.com",
            ...     "pass_sender",
            ...     "receiver@email.com",
            ...     "subject",
            ...     "text_message",
            ...     "<div>html_message<div>",
            ... )
            >>> print(my_instance.email_sender)
            sender@email.com
            >>> my_instance.email_sender = "new_email_valid@email.com"
            >>> print(my_instance.email_sender)
            new_email_valid@email.com
            >>> my_instance.email_sender = "invalid_email"
            Traceback:
                ValueError: The sender mail is not valid mail. Given: invalid_email
        """
        return self._email_sender

    @email_sender.setter
    def email_sender(self, value: str):
        if IS_EMAIL()(value)[1] is None:
            self._email_sender = value
        else:
            raise ValueError("The sender mail is not valid mail. Given: {0}".format(value))

    @property
    def email_receiver(self) -> str:
        """Get ot Set the email receiver.

        :GET:

            Get the email receiver

        :SET:
            When adding an email, it is validated, if it is not a valid email it returns a ValueError

        Example:
            >>> my_instance = MailSender(
            ...     "sender@email.com",
            ...     "pass_sender",
            ...     "receiver@email.com",
            ...     "subject",
            ...     "text_message",
            ...     "<div>html_message<div>",
            ... )
            >>> print(my_instance.email_receiver)
            receiver@email.com
            >>> my_instance.email_receiver = "new_email_valid@email.com"
            >>> print(my_instance.email_sender)
            new_email_valid@email.com
            >>> my_instance.email_sender = "invalid_email"
            Traceback:
                ValueError: The receiver mail is not valid mail. Given: invalid_email
        """
        return self._email_receiver

    @email_receiver.setter
    def email_receiver(self, value: str):
        if IS_EMAIL()(value)[1] is None:
            self._email_receiver = value
        else:
            raise ValueError("The receiver mail is not valid mail. Given: {0}".format(value))

    @property
    def alternative_sender(self) -> str:
        """The alternative sender is optional, it is the reply email.
        :GET:

            Get the alternative sender

        :SET:

            When adding an email, it is validated, if it is not a valid email it returns a ValueError
        """
        return self._alternative_sender

    @alternative_sender.setter
    def alternative_sender(self, value: str):
        if value is not None:
            if IS_EMAIL()(value)[1] is None:
                self._alternative_sender = value
            else:
                raise ValueError("The alternative mail is not valid mail. Given: {0}".format(value))
        else:
            self._alternative_sender = None

    def send(self):
        """This method sends the email that was generated.
        """
        message = MIMEMultipart("alternative")
        message["Subject"] = self.subject
        message["From"] = self.alternative_sender or self.email_sender
        message["To"] = self.email_receiver
        text = MIMEText(self.text_message, "plain")
        html = MIMEText(self.html_message, "html")
        message.attach(text)
        message.attach(html)
        if self.use_ssl:
            host = smtplib.SMTP_SSL(self.server, self.port)
        else:
            host = smtplib.SMTP(self.server, self.port)
        if self.use_tls:
            host.starttls()

        host.login(self.email_sender, self.password_sender)
        host.sendmail(
            self.email_sender, self.email_receiver, message.as_string()
        )



from org.transcrypt.stubs.browser import __pragma__
from phanterpwa.frontend import helpers
__pragma__('alias', "jQuery", "$")
__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = String = setTimeout = Date = RegExp = \
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = Date = 0

__pragma__('noskip')


DIV = helpers.XmlConstructor.tagger("div")


class ProgressBar():
    """docstring for ProgressBar"""
    def __init__(self, id_container):
        self.element = jQuery(id_container)
        self.events_set = set()
        self.start()

    def addEventProgressBar(self, event_name):
        self.events_set.add(event_name)

    def removeEventProgressBar(self, event_name):
        try:
            self.events_set.remove(event_name)
        except:
            if window.PhanterPWA.DEBUG:
                console.info("Error on remove event of progressbar")

    def forceStop(self):
        self.events_set = set()
        self.element.find(".phanterpwa-main_progressbar-wrapper").removeClass("enabled")

    def check_progressbar(self):
        if len(self.events_set) == 0:
            self.element.find(".phanterpwa-main_progressbar-wrapper").removeClass("enabled")
        else:
            self.element.find(".phanterpwa-main_progressbar-wrapper").addClass("enabled")


    def start(self):
        self.element.html(self.xml())
        window.setInterval(self.check_progressbar, 30)

    def xml(self):
        return DIV(
            DIV(
                DIV(
                    _class="phanterpwa-main_progressbar-movement",
                ),
                _class="phanterpwa-main_progressbar"
            ),
            _class="phanterpwa-main_progressbar-wrapper enabled"
        ).jquery()
